# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimString,
    HprimDate,
    HprimForeign,
)
from driverng.utils import num_years


class KiestraSegmentH(HeaderSegment):
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    orders = HprimForeign("O", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KiestraSegmentP(Segment):
    patient_id = HprimString((4, 1))
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    # Champ P15
    special_field1 = HprimString(15)
    # Champ P16
    special_field2 = HprimString(16)
    # Champ P20
    active_medication = HprimString(20)
    # Champ P22
    practice_field1 = HprimString(22)
    # Champ P23
    practice_field2 = HprimString(23)
    location = HprimString(26)

    orders = HprimForeign("O", "patient")


class KiestraSegmentO(Segment):
    accession_number = HprimString((3, 1))
    accession_no_ref = HprimString((3, 2))
    test_id = HprimString((5, 4))
    collection_date = HprimDate(8)
    action_code = HprimString(12)
    receipt_date = HprimDate(15)
    specimen_type = HprimString((16, 1))
    users_field_no1a = HprimString((19, 1))
    users_field_no1b = HprimString((19, 2))
    users_field_no2a = HprimString((20, 1))
    users_field_no2b = HprimString((20, 2))

    results = HprimForeign("R", "order")


class KiestraSegmentR(Segment):
    result_type_code = HprimString((3, 4))
    consumable_sequence_number = HprimString((3, 5))
    # No d'isolat
    carrier_seq_no = HprimString((3, 6))
    carrier_comment = HprimString((3, 7))
    test_id = HprimString((4, 1))
    result_value1 = HprimString((4, 2))
    result_value2 = HprimString((4, 3))
    result_value3 = HprimString((4, 4))
    audit = HprimString(11)
    datetime_test = HprimDate((13, 1))
    datetime_test2 = HprimDate((13, 2))
    instrument_id = HprimString(14)


class KiestraSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimString(4)


class KiestraSegmentQ(Segment):
    accession_number = HprimString((3, 2))


class KiestraSegmentL(Segment):
    code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "kiestra",
        "label": "Kiestra BD"
    }
    SEGMENT_CLASS_MAP = {
        "H": KiestraSegmentH,
        "P": KiestraSegmentP,
        "O": KiestraSegmentO,
        "R": KiestraSegmentR,
        "C": KiestraSegmentC,
        "Q": KiestraSegmentQ,
        "L": KiestraSegmentL,
    }
    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "kiestra_liste_germes",
            label="Liste des codes germes Kiestra à intégrer dans IDENT2 séparés par ;",
        ),
        AstmMessageV2.IntegerOption(
            "kiestra_specif_patient_id",
            label="Spécification sur le champ du No Id patient (P.4)",
            choices=[
                (1, "Id MCA est transmis dans le champ P.4"),
                (2, "Transmettre le nom et le prénom du patient dans le champ P4 à la place du no Id patient"),
                (3, "Spécification application Urine de BD No echantillon dans P.4 et nom prénom patient dans O.20.1"),
            ],
            default="1"
        ),
    ]

    def __init__(
            self,
            kiestra_liste_germes="",
            kiestra_specif_patient_id="1",
            **kwargs
    ):
        self.liste_germes = kiestra_liste_germes
        self.specif_patient_id = kiestra_specif_patient_id
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = KiestraSegmentH(type_segment="H", **kwargs)
        message.end = KiestraSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            action={"del": worklist.IGNORE, worklist.ELSE: "N"},
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
        ):
            age = ""
            age = num_years(patient["birthdate"], patient["collection_date"])
            if age is None:
                age = ""
            seg_p = msg.patients.add(
                birthdate=patient.birthdate,
                sex=patient.sex,
                location=patient.location
            )
            if self.specif_patient_id == "2":
                patient_name = "{} {}".format(patient.last_name, patient.first_name)
                seg_p.patient_id = "{:.20s}".format(patient_name)
            elif self.specif_patient_id == "1":
                seg_p.patient_id = patient.patient_id
                seg_p.last_name = patient.last_name
                seg_p.first_name = patient.first_name
            for sample, action, priority in patient.samples:
                for order in sample.orders:
                    # L'envoi se fait par boite et par code demande
                    if sample.media_code:
                        for i, media_code in enumerate(sample.media_code.split("_"), 0):
                            for result in sample.results:
                                if "P15" in result.code:
                                    seg_p.special_field1 = result.value
                                elif "P16" in result.code:
                                    seg_p.special_field2 = result.value
                                elif "LEUCO" in result.code:
                                    seg_p.special_field2 = result.value
                                elif "P20" in result.code:
                                    seg_p.active_medication = result.value
                                elif "P22" in result.code:
                                    seg_p.practice_field1 = result.value
                                elif "P23" in result.code:
                                    seg_p.practice_field2 = result.value
                            if (
                                    "P15" in order.code or
                                    "P16" in order.code or
                                    "P20" in order.code or
                                    "P22" in order.code or
                                    "P23" in order.code or
                                    "LEUCO" in order.code
                            ):
                                continue
                            seg_o = seg_p.orders.add(
                                accession_number=sample.sample_id,
                                test_id=order.code,
                                action_code=action,
                                specimen_type=sample.specimen,
                                collection_date=sample.collection_date,
                                receipt_date=sample.request_date,
                                users_field_no1b=media_code
                            )
                            if self.specif_patient_id == "3":
                                seg_p.patient_id = sample.sample_id
                                seg_o.users_field_no2a = "{} {}".format(patient.last_name, patient.first_name)
                            list_germes = []
                            for result in sample.results:
                                if "germes" in result["resu_json"]:
                                    for germe in result["resu_json"]["germes"]:
                                        if "id" in germe:
                                            list_germes.append(germe.get("id"))
                                        elif "Id" in germe:
                                            list_germes.append(germe.get("Id"))
                                if list_germes:
                                    for germe in list_germes:
                                        seg_o.results.add(
                                            result_type_code="ORG",
                                            # Name of organism
                                            test_id=germe,
                                            # The amount of growth (Quantity)
                                            result_value1=result.value,
                                        )
                    # Aucune boite, l'envoi se fait par code demande
                    else:
                        for result in sample.results:
                            if "P15" in result.code:
                                seg_p.special_field1 = result.value
                            elif "P16" in result.code:
                                seg_p.special_field2 = result.value
                            elif "LEUCO" in result.code:
                                seg_p.special_field2 = result.value
                            elif "P22" in result.code:
                                seg_p.practice_field1 = result.value
                            elif "P23" in result.code:
                                seg_p.practice_field2 = result.value
                        if (
                                "P15" in order.code or
                                "P16" in order.code or
                                "P22" in order.code or
                                "P23" in order.code or
                                "LEUCO" in order.code
                        ):
                            continue
                        seg_o = seg_p.orders.add(
                            accession_number=sample.sample_id,
                            test_id=order.code,
                            action_code=action,
                            specimen_type=sample.specimen,
                            collection_date=sample.collection_date,
                            receipt_date=sample.request_date,
                        )
        return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.accession_number,
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                list_barcode_media = []
                barcode_media = False
                comments = []
                for seg_o in seg_p.orders:
                    sample = worklist.get_or_create(
                        type="patient",
                        sample_id=seg_o.accession_number,
                    )
                    if seg_o.users_field_no2b:
                        list_barcode_media.append(seg_o.users_field_no2b)
                        sample.barcode_media = list_barcode_media
                        comments.append(seg_o.users_field_no1a)
                        barcode_media = True
                    specimen_code = ""
                    code_identification = ""
                    code_identification2 = ""
                    liste_identification_germes = []
                    liste_identification2_germes = []
                    liste_atb = []
                    code_germe = ""
                    nbr_qty = 1
                    for seg_r in seg_o.results:
                        barcode_media = False
                        if seg_r.result_type_code not in ('ORG', 'IT', 'RES', 'AB', 'STREAK', 'SPM'):
                            specimen_code = seg_r.result_type_code
                        elif seg_r.result_type_code == "ORG":
                            sample.results.add(
                                code="{}ID{}{}".format(specimen_code, nbr_qty, seg_o.test_id),
                                value=seg_r.test_id
                            )
                            code_identification = "IDENT_{}{}".format(specimen_code, seg_o.test_id)
                            liste_identification_germes.append({
                                "rang": seg_r.consumable_sequence_number,
                                "id": seg_r.test_id,
                                "qty": seg_r.result_value1
                            })
                            if seg_r.test_id in self.liste_germes:
                                code_identification2 = "IDENT2_{}{}".format(specimen_code, seg_o.test_id)
                                liste_identification2_germes.append({
                                    "rang": seg_r.consumable_sequence_number,
                                    "id": seg_r.test_id,
                                    "qty": seg_r.result_value1
                                })
                            else:
                                code_identification2 = "IDENT2_{}{}".format(specimen_code, seg_o.test_id)
                                liste_identification2_germes.append({
                                    "rang": seg_r.consumable_sequence_number,
                                    "info": seg_r.test_id,
                                    "qty": seg_r.result_value1
                                })
                            code_germe = "G_{}".format(seg_r.test_id)
                            if seg_r.result_value1:
                                sample.results.add(
                                    code="{}QT{}{}".format(specimen_code, nbr_qty, seg_o.test_id),
                                    value=seg_r.result_value1
                                )
                            nbr_qty += 1
                        elif seg_r.result_type_code == "IT":
                            sample.results.add(
                                code="{}{}{}".format(specimen_code, seg_r.test_id, seg_o.test_id),
                                value=seg_r.result_value1
                            )
                        elif seg_r.result_type_code == "AB":
                            code_antibio = "A_{}".format(seg_r.test_id)
                            liste_atb.append({
                                "antibiotic": code_antibio,
                                "mic": seg_r.result_value1,
                                "sensitivity": seg_r.result_value3
                            })
                        elif seg_r.result_type_code == "RES":
                            sample.results.add(
                                code="{}_{}".format(seg_o.test_id, seg_r.test_id),
                                value=seg_r.result_value1
                            )
                        elif seg_r.result_type_code == "STREAK":
                            sample.results.add(
                                code=seg_r.result_type_code,
                                value=seg_r.test_id,
                                date=seg_r.datetime_test2,
                            )
                        elif seg_r.result_type_code == "SPM":
                            sample_comments = []
                            for seg_c in seg_r.commentaires:
                                if seg_c.comment_text:
                                    sample_comments.append(seg_c.comment_text)
                            if sample_comments:
                                sample.results.add(
                                    code="{}_COM".format(seg_o.test_id),
                                    value=";".join(sample_comments)
                                )
                    if code_identification and liste_identification_germes:
                        sample.results.add(
                            code=code_identification,
                            value={
                                "seq": "0",
                                "type_resultat": "identification_germes",
                                "germes": liste_identification_germes
                            },
                            instrument=seg_r.instrument_id
                        )
                    if code_identification2 and liste_identification2_germes:
                        sample.results.add(
                            code=code_identification2,
                            value={
                                "seq": "0",
                                "type_resultat": "identification_germes",
                                "germes": liste_identification2_germes
                            },
                            instrument=seg_r.instrument_id
                        )
                    if liste_atb:
                        sample.results.add(
                            code=specimen_code,
                            value=[{
                                "antibiogram": liste_atb,
                                "value": code_germe,
                            }],
                            date=seg_r.datetime_test,
                            instrument=seg_r.instrument_id
                        )
                if barcode_media and list_barcode_media:
                    sample.results.add(
                        code="ID_BOITE",
                        value=";".join(list_barcode_media),
                        comments=comments
                    )
        return worklist
